%define anolis_release 3
%global toolchain clang

%undefine _include_frame_pointers

%global maj_ver 17
%global min_ver 0
%global patch_ver 6
%global compiler_rt_version %{maj_ver}.%{min_ver}.%{patch_ver}

%global crt_srcdir compiler-rt-%{compiler_rt_version}.src
%global cmake_srcdir cmake-%{compiler_rt_version}.src

# see https://sourceware.org/bugzilla/show_bug.cgi?id=25271
%global optflags %(echo %{optflags} -D_DEFAULT_SOURCE)

# see https://gcc.gnu.org/bugzilla/show_bug.cgi?id=93615
%global optflags %(echo %{optflags} -Dasm=__asm__)

Name:		compiler-rt
Version:	%{compiler_rt_version}
Release:	%{anolis_release}%{?dist}
Summary:	LLVM "compiler-rt" runtime libraries

License:	Apache-2.0 WITH LLVM-exception OR NCSA OR MIT
URL:		http://llvm.org
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{compiler_rt_version}/%{crt_srcdir}.tar.xz
Source3:        https://github.com/llvm/llvm-project/releases/download/llvmorg-%{compiler_rt_version}/%{cmake_srcdir}.tar.xz

# Patches for LoongArch
Patch1:    0001-sanitizer-msan-VarArgHelper-for-loongarch64.patch
Patch2:    0002-CFI-Allow-LoongArch-67314.patch
Patch3:    0003-test-compiler-rt-Mark-several-tests-as-UNSUPPORTED-o.patch
Patch4:    0004-tsan-Add-support-for-linux-loongarch64-in-lib-tsan-g.patch
Patch5:    0005-tsan-Refine-fstat-64-interceptors-86625.patch

BuildRequires:	clang
BuildRequires:	cmake
BuildRequires:	ninja-build
BuildRequires:	python3
BuildRequires:	python3-devel
BuildRequires:	llvm-devel
BuildRequires:  zlib-devel

BuildRequires:  llvm-googletest

Requires: clang-resource-filesystem

%description
The compiler-rt project is a part of the LLVM project. It provides
implementation of the low-level target-specific hooks required by
code generation, sanitizer runtimes and profiling library for code
instrumentation, and Blocks C language extension.

%package doc
Summary:        Documentation for %{name}
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentations for %{name}

%prep
%setup -T -q -b 3 -n %{cmake_srcdir}
cd ..
mv %{cmake_srcdir} cmake
%autosetup -n %{crt_srcdir} -p2

%py3_shebang_fix lib/hwasan/scripts/hwasan_symbolize

%build
export ASMFLAGS=$CFLAGS

%cmake	-GNinja \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_MODULE_PATH=%{_libdir}/cmake/llvm \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	\
	-DLLVM_LIBDIR_SUFFIX=64 \
	-DCOMPILER_RT_INCLUDE_TESTS:BOOL=OFF

%cmake_build

%install
%cmake_install

# move blacklist/abilist files to where clang expect them
mkdir -p %{buildroot}%{_libdir}/clang/%{maj_ver}/share
mv -v %{buildroot}%{_datadir}/*list.txt  %{buildroot}%{_libdir}/clang/%{maj_ver}/share/

# move sanitizer libs to better place
%global libclang_rt_installdir lib/linux
mkdir -p %{buildroot}%{_libdir}/clang/%{maj_ver}/lib
mv -v %{buildroot}%{_prefix}/%{libclang_rt_installdir}/*_rt* %{buildroot}%{_libdir}/clang/%{maj_ver}/lib
mkdir -p %{buildroot}%{_libdir}/clang/%{maj_ver}/lib/linux/
pushd %{buildroot}%{_libdir}/clang/%{maj_ver}/lib
for i in *.a *.so
do
        ln -s ../$i linux/$i
done

popd

%files
%license LICENSE.TXT
%{_includedir}/*
%{_libdir}/clang/%{maj_ver}/lib/*
%{_libdir}/clang/%{maj_ver}/share/*
%ifnarch loongarch64
%{_bindir}/hwasan_symbolize
%endif

%files doc
%doc README.txt CODE_OWNERS.TXT docs/TestingGuide.rst

%changelog
* Tue Apr 9 2024 Chen Li <chenli@loongson.cn> - 17.0.6-3
- LoongArch Backport: Improve support and fix some bugs for compiler-rt

* Mon Apr 1 2024 Wenlong Zhang <zhangwenlong@loongson.cn> - 17.0.6-2
- build for loongarch64

* Thu Mar 21 2024 mgb01105731 <mgb01105731@alibaba-inc.com> - 17.0.6-1
- update to 17.0.6

* Thu Aug 17 2023 Chen Li <chenli@loongson.cn> - 16.0.6-2
- Backport LoongArch patches from llvmorg-17.0.0-rc2
- Exclude hwasan_symbolize on loongarch64 because of nonsupport

* Tue Aug 8 2023 Funda Wang <fundawang@yeah.net> - 16.0.6-1
- New version 16.0.6

* Thu May 18 2023 Funda Wang <fundawang@yeah.net> - 16.0.4-1
- New version 16.0.4

* Fri Apr 14 2023 Funda Wang <fundawang@yeah.net> - 16.0.1-1
- New version 16.0.1

* Wed Feb 1 2023 Funda Wang <fundawang@yeah.net> - 15.0.7-1
- New version 15.0.7

* Sun Jul 31 2022 happy_orange <songnannan@linux.alibaba.com> - 13.0.1-1
- Init pcakage from upstream
